#!/usr/bin/perl -I/usr/sausalito/perl
#
# $Id: savechanges.pl,v 1.0.0.0 Sat 21 Jun 2008 08:25:02 AM BST mstauber Exp $
# Copyright 2006-2008 Solarspeed Ltd. and NuOnce Networks, Inc. All rights reserved.
#
# Handler which writes / updates the drbd and HA config files.
#

use CCE;
use POSIX qw(isalpha);
use Data::Dumper;
use Sauce::Service;
use updateCluster;

my $DEBUG = 0;
$DEBUG && open(STDERR, ">/tmp/cluster_savechanges.debug");
$DEBUG && warn `date`;

my $cce = new CCE('Domain' => 'base-vserver');

$cce->connectfd();

$drbd_conf = '/etc/drbd.d/disk.res';
$ha_conf = '/etc/ha.d/ha.cf';
$haresources_conf = '/etc/ha.d/haresources';

$eventObj = $cce->event_object();
$eventNewObj = $cce->event_new();
$DEBUG && warn Dumper($eventObj);
$DEBUG && warn Dumper($eventNewObj);

if ($eventObj->{CLASS} eq "Network") {
    ($clusterOid) = $cce->find('Cluster', '');

    $DEBUG && warn "Clusteroid: " . $clusterOid ."\n";
    $networkInterfaces = updateCluster::getNetworkInterfaces($cce);
    $IP_eth0 = $networkInterfaces->{'br0'};
    $IP_eth1 = $networkInterfaces->{'eth1'};

    if($clusterOid < 1) {
	$DEBUG && warn "Network changed and no cluster object, creating new!\n";

	if ( $IP_eth1 ) {
	    $DEBUG && "Created Cluster configuration\n";
	    updateCluster::createClusterConf($cce,$IP_eth0, $IP_eth1, 0);
	} 
	else {
	    $DEBUG && "Created Cluster configuration\n";
	    updateCluster::createClusterConf($cce,$IP_eth0, "127.0.0.1", 0);
	}
	$cce->bye('SUCCESS');
	exit(0);
    } 
#    else {
#	if(eventNewObj->{enabled} eq "0" && $eventObj->{device} eq "eth1") {
#	    $DEBUG && warn "Secondary interface deleted, turn off cluster software.\n";
#	    ($ok) = $cce->set($clusterOid, '', {'enabled' => '0'});
#	    Sauce::Service::service_run_init("heartbeat", "stop");
#	    Sauce::Service::service_set_init('heartbeat', 'off');
#	    $cce->bye('SUCCESS');
#	    exit(0);
#	}
#	if(eventObj->{device} eq "br0" && $IP_eth1 eq "") {
#	    $DEBUG && warn "Secondary interface not configured, exit.\n";
#	    $cce->bye('SUCCESS');
#	    exit(0);
#	}
#    }

    $DEBUG && warn Dumper($eventObj);
    $DEBUG && warn Dumper($eventNewObj);

    ($ok, $clusterObj) = $cce->get($clusterOid);
    if ($eventObj->{device} eq "br0") {
	$clusterObj->{my_eth0} = $eventNewObj->{ipaddr};
    }
    else {
	if ($eventNewObj->{ipaddr}) {
	    $clusterObj->{my_eth1} = $eventNewObj->{ipaddr};
	}
	else {
	    $clusterObj->{my_eth1} = "127.0.0.1";
	}
    }
    ($ok) = $cce->set($clusterOid, '', $clusterObj);
}
else {
    $clusterObj = $eventObj;
}

$Data::Dumper::Indent = 3;
$DEBUG && warn Dumper($clusterObj);
$DEBUG && warn "\n";
# Do the deeds:
cluster_write_conf($clusterObj);

$enable = $clusterObj->{enabled};
$bulldozer = $clusterObj->{bulldozer};
$DEBUG && warn "Cluster enabled: $enable - dozer: $bulldozer\n";

if( $enable eq "1") {
    $DEBUG && warn "Enabled\n";
    $DEBUG && warn "Reloading drbd\n";
    if ($bulldozer ne "") {
	system("/sbin/drbdadm disconnect all > /dev/null 2>&1");
	system("/sbin/drbdadm invalidate all > /dev/null 2>&1");
	system("/etc/init.d/drbd stop > /dev/null 2>&1");
	system("/usr/bin/yes yes | /sbin/drbdadm create-md all > /dev/null 2>&1");
	system("/etc/init.d/drbd start > /dev/null 2>&1");
	#system("/sbin/drbdadm -- --overwrite-data-of-peer primary all > /dev/null 2>&1");
	system("/sbin/drbdadm connect all > /dev/null 2>&1");
    }
    Sauce::Service::service_run_init("drbd", "reload");
    Sauce::Service::service_set_init('drbd', 'on');
    $DEBUG && warn "Reloading heartbeat\n";
    Sauce::Service::service_run_init("heartbeat", "restart");
    Sauce::Service::service_set_init('heartbeat', 'on');
} else { 
   if( Sauce::Service::service_run_init('heartbeat', 'status') =~ /heartbeat OK/g) {
	Sauce::Service::service_run_init("heartbeat", "stop");
      }
    Sauce::Service::service_set_init('heartbeat', 'off');
    Sauce::Service::service_run_init("heartbeat", "stop");
    if( Sauce::Service::service_run_init('drbd', 'status') =~ /loaded OK/g) {
	Sauce::Service::service_run_init("drbd", "stop");
	system("/etc/init.d/drbd stop");
      }
    system("/etc/init.d/drbd reload");
    Sauce::Service::service_set_init('drbd', 'off');
}

# We need to say 'SUCCESS' or CCE will bitch
$cce->bye('SUCCESS');
exit(0);

# Use GUI submitted new individual values:
sub cluster_write_conf {
   
    $system = shift;

    $wfc_timeout      = $system->{'wfc_timeout'};
    $degr_wfc_timeout = $system->{'degr_wfc_timeout'};
    $rate             = $system->{'rate'};
    $Std_Pri_fqdn     = $system->{'Std_Pri_fqdn'};
    $Std_Pri_Device   = $system->{'Std_Pri_Device'};
    $Std_Pri_Disk     = $system->{'Std_Pri_Disk'};
    $my_eth1          = $system->{'my_eth1'};
    $Std_Sec_fqdn     = $system->{'Std_Sec_fqdn'};
    $Std_Sec_Device   = $system->{'Std_Sec_Device'};
    $Std_Sec_Disk     = $system->{'Std_Sec_Disk'};
    $peer_eth1        = $system->{'peer_eth1'};
    $keepalive        = $system->{'keepalive'};
    $warntime         = $system->{'warntime'};
    $deadtime         = $system->{'deadtime'};
    $initdead         = $system->{'initdead'};
    $peer_eth0        = $system->{'peer_eth0'};
    $auto_failback    = $system->{'auto_failback'};
    $makeprimary      = $system->{'makeprimary'};
    $ping             = $system->{'ping'};

    if($system->{'makeprimary'} == "") {
	$haresources_fqdn = $Std_Sec_fqdn;
    } else {
	$haresources_fqdn  = $Std_Pri_fqdn;
    }

    if ($haresources_fqdn eq "localhost") {
	$primary_line = "";
    }
    else {
	$primary_line = "become-primary-on $haresources_fqdn;";
    }

    ($clusterOid) = $cce->find('Cluster', '');
    ($ok) = $cce->set($clusterOid, '', { 'initialdrbd' => "$haresources_fqdn", 'bulldozer' => "" });

    @myfqdn = split(/\./, $Std_Pri_fqdn);
    $myhost = $myfqdn[0];
    
    # DRBD conf
    &newconf;

    if ($peer_eth1 eq "127.0.0.1") {
	$bcast_line = "bcast lo";
    }
    else {
	$bcast_line = "
ucast br0 $peer_eth0
ucast eth1 $peer_eth1";
    }

    open(CONF, ">$ha_conf") || die "Can't write to $ha_conf!";
    print CONF "#
# Heartbeat configuration:
#

logfacility daemon

# Heartbeat cluster members
node $Std_Pri_fqdn
node $Std_Sec_fqdn

# Heartbeat communication timing
keepalive $keepalive
warntime $warntime
deadtime $deadtime
initdead $initdead

# Heartbeat communication paths
udpport 694
$bcast_line
#baud 19200
#serial /dev/ttyS1

# Fail back automatically
auto_failback $auto_failback

# Monitoring of network connection to default gateway
ping $ping
respawn hacluster /usr/lib64/heartbeat/ipfail
";
    close CONF;
    
    open(CONF, ">$haresources_conf") || die "Can't write to $haresources_conf!";
    print CONF "$haresources_fqdn drbddisk::vz Filesystem::/dev/drbd0::/vz::ext3 WatchDog libvirtd libvirt-guests vzeventd vz";
    close CONF;
}


#
## New subs:
#

# Write drbd.conf for 0.8:

sub newconf {
        open(CONF, ">$drbd_conf") || die "Can't write to $drbd_conf!";
        print CONF "
resource vz {

        protocol C;

        handlers {

            # what should be done in case the node is primary, degraded
            # (=no connection) and has inconsistent data.
            pri-on-incon-degr \"echo o > /proc/sysrq-trigger ; halt -f\";

            # The node is currently primary, but lost the after split brain
            # auto recovery procedure. As as consequence it should go away.
            pri-lost-after-sb \"echo o > /proc/sysrq-trigger ; halt -f\";

            # In case you have set the on-io-error option to 'call-local-io-error',
            # this script will get executed in case of a local IO error. It is
            # expected that this script will case a immediate failover in the
            # cluster.
            local-io-error \"echo o > /proc/sysrq-trigger ; halt -f\";

        }

        disk {
            on-io-error   call-local-io-error;
        }

        syncer {
            rate $rate;
            al-extents 257;
        }

        net {
            after-sb-0pri discard-younger-primary;
            after-sb-1pri consensus;
            after-sb-2pri call-pri-lost-after-sb;
        }

	startup {
	    wfc-timeout $wfc_timeout;
	    degr-wfc-timeout $degr_wfc_timeout;
	    $primary_line
        }

	on $Std_Pri_fqdn {
	    device     $Std_Pri_Device;
	    disk       $Std_Pri_Disk;
	    address    $my_eth1:7788;
	    meta-disk  internal;
	}

	on $Std_Sec_fqdn {
	    device     $Std_Sec_Device;
	    disk       $Std_Sec_Disk;
	    address    $peer_eth1:7788;
	    meta-disk  internal;
	}
}
";
	close CONF;
}


# We need to say 'SUCCESS' or CCE will bitch:
$cce->bye('SUCCESS');
exit(0);


