#
# cluster.pm
#
# Copyright 2011 Bluapp AB,  All rights reserved.
#
# A library to handle common cluster code.

package updateCluster;

#use diagnostics;
use lib '/usr/sausalito/perl';
use CCE;

###
# Check if both network interfaces are configured. Return the IP-address of the interfaces.
###
sub getNetworkInterfaces
{
    $cce = shift;
    $DEBUG = shift;
    if ($DEBUG){ use Sys::Syslog qw( :DEFAULT setlogsock); }
    @networkOids = $cce->findx("Network", {}, {}, "ascii" ,"device");
    for $networkOid (@networkOids) {
	($ok, $netObj) = $cce->get($networkOid);
	if($netObj->{device} eq "br0") {
	    $IP_eth0 = $netObj->{ipaddr};
	} else {
	    if($netObj->{enabled}) {
		$IP_eth1 = $netObj->{ipaddr};
	    }
	}
    }
    $networkInterfaces = { 
	'br0' => $IP_eth0,
	'eth1' => $IP_eth1
	};
    return $networkInterfaces;
}

sub getClusterStatus
{
    $DEBUG = shift;
    if ($DEBUG){ use Sys::Syslog qw( :DEFAULT setlogsock); }

    &check_if_cluster($cce, $DEBUG);

# Location of the drbd.conf file:
    $cfgfile = '/etc/drbd.d/disk.res';

# Location of the ha.cf file:
    $hacfgfile = '/etc/ha.d/ha.cf';
    
# Location of the drbd proc::
    $drbdproc = '/proc/drbd';
    
    if(open (FD, $cfgfile)) {
	while ($line = <FD>) {
	    chomp($line);
	    next if $line =~ /^\s*$/; 		# skip blank lines
	    if ($line =~ /wfc-timeout (.*)\;/) {
		$wfc_timeout = $1;
	    }
	    if ($line =~ /degr-wfc-timeout (.*)\;/) {
		$degr_wfc_timeout = $1;
	    }
	    if ($line =~ /on-io-error (.*);/) {
		$on_io_error = $1;
	    }
	    if ($line =~ /rate (.*)M;/) {
		$rate = $1 . "M";
	    }
	    if (($line =~ /on (.*) \{/) && (!$parsed_Std_Sec_fqdn)) {
		$parsed_Std_Pri_fqdn = $1;
	    }
	    if (($line =~ /on (.*) \{/) && ($parsed_Std_Pri_fqdn)) {
		$parsed_Std_Sec_fqdn = $1;
	    }
	    if (($line =~ /device(.*)\/dev\/(.*);/) && (!$parsed_Std_Pri_Device)) {
		$parsed_Std_Pri_Device = "/dev/" . $2;
	    }
	    if (($line =~ /device(.*)\/dev\/(.*);/) && ($parsed_Std_Pri_Device)) {
		$parsed_Std_Sec_Device = "/dev/" . $2;
	    }
	    if (($line =~ /disk(.*)\/dev\/(.*);/) && (!$parsed_Std_Pri_Disk)) {
		$parsed_Std_Pri_Disk = "/dev/" . $2;
	    }
	    if (($line =~ /disk(.*)\/dev\/(.*);/) && ($parsed_Std_Pri_Disk)) {
		$parsed_Std_Sec_Disk = "/dev/" . $2;
	    }
	    if (($line =~ /address\s{0,10}(.*):/) && (!$parsed_Std_Pri_ipAddr)) {
		$parsed_Std_Pri_ipAddr = $1;
	    }
	    if (($line =~ /address\s{0,10}(.*):/) && ($parsed_Std_Pri_ipAddr)) {
		$parsed_Std_Sec_ipAddr = $1;
	    }
	} 
	close(FD);
    } else {
	## /etc/ha.d/ha.cf is not created
	&debug_msg("/etc/drbd.d/disk.res is not created, configure cluster service. \n");
	$cce->bye('FAIL', '/etc/drbd.d/disk.res is not created, configure cluster service.');
	exit(1);
    }

    &debug_msg("wfc_timeout          : $wfc_timeout \n");
    &debug_msg("wfc_timeout          : $wfc_timeout \n");
    &debug_msg("degr_wfc_timeout     : $degr_wfc_timeout \n");
    &debug_msg("rate                 : $rate \n");
    &debug_msg("parsed_Std_Pri_fqdn  : $parsed_Std_Pri_fqdn \n");
    &debug_msg("parsed_Std_Sec_fqdn  : $parsed_Std_Sec_fqdn \n");
    &debug_msg("parsed_Std_Pri_Device: $parsed_Std_Pri_Device \n");
    &debug_msg("parsed_Std_Sec_Device: $parsed_Std_Sec_Device \n");
    &debug_msg("parsed_Std_Pri_Disk  : $parsed_Std_Pri_Disk \n");
    &debug_msg("parsed_Std_Sec_Disk  : $parsed_Std_Sec_Disk \n");
    &debug_msg("parsed_peer_eth1     : $parsed_Std_Sec_ipAddr \n");
    
    if(open (FD, $drbdproc)) {
	while ($line = <FD>) {
	    chomp($line);
	    
	    &debug_msg("$line \n");

	    next if $line =~ /^\s*$/; 		# skip blank lines
	    if ($line =~ / 0: cs:(.*) ro:(.*) ds:(.*) (C){0,1} /) {
		$ConStatus = $1;
		$SyncStatus = $2;
		$LinkDiagnostic = $3;
	    }
	    
	    if ($line =~ /\[(.*)\] sync\'ed: (.*) \((.*)\/(.*)\)M/) {
		$sync_progress = "[" . $1 . "]";
		$sync_percentage = $2;
		$sync_current = $3;
		$sync_max = $4;
	    }
	    if ($line =~ /finish: (.*) speed: (.*)/) {
		$sync_finish = $1;
		$sync_speed = $2;
	    }
	}
	close(FD);
    } else {
	## DRBD is not started
	&debug_msg("DRBD is not running at this time. Configure cluster service if required. \n");
	$cce->bye('SUCCESS', 'DRBD is not running at this time. Configure cluster service if required.');
	exit(0);
    }
    if (($sync_progress == "") && ($sync_current == "") && ($sync_max == "") && ($sync_max == "")) {
	$synchronizing = "0";
    }
    else {
	$synchronizing = "1";
    }
    
    &debug_msg("\n");
    &debug_msg("ConStatus       : $ConStatus \n");
    &debug_msg("SyncStatus      : $SyncStatus \n");
    &debug_msg("LinkDiagnostic  : $LinkDiagnostic \n");
    &debug_msg("sync_progress   : $sync_progress \n");
    &debug_msg("sync_percentage : $sync_percentage \n");
    &debug_msg("sync_current    : $sync_current \n");
    &debug_msg("sync_max        : $sync_max \n");
    &debug_msg("sync_finish     : $sync_finish \n");
    &debug_msg("sync_speed      : $sync_speed \n");
    &debug_msg("synchronizing   : $synchronizing \n");
    

    if(open (FD, $hacfgfile)) {
	while ($line = <FD>) {
	    chomp($line);
	    next if $line =~ /^\s*$/;               # skip blank lines
	    if ($line =~ /auto_failback (.*)/) {
		$auto_failback = $1;
	    }
	    if ($line =~ /keepalive (.*)/) {
		$keepalive = $1;
	    }
	    if ($line =~ /warntime (.*)/) {
		$warntime = $1;
	    }
	    if ($line =~ /deadtime (.*)/) {
		$deadtime = $1;
	    }
	    if ($line =~ /initdead (.*)/) {
		$initdead = $1;
	    }
	    if ($line =~ /ping (.*)/) {
		$ping = $1;
	    }
	    if (($line =~ /ucast eth1 (.*)/) && (!$parsed_ha_peer_eth1)) {
		$parsed_ha_peer_eth1 = $1;
	    }
	    if (($line =~ /ucast br0 (.*)/) && (!$parsed_ha_peer_eth0)) {
		$parsed_ha_peer_eth0 = $1;
	    }
	}
	close(FD);
    } else {
	## /etc/ha.d/ha.cf is not created
	&debug_msg("/etc/ha.d/ha.cf is not created, configure cluster service. \n");
	$cce->bye('FAIL', '/etc/ha.d/ha.cf is not created, configure cluster service.');
	exit(1);

    }


    &debug_msg("auto_failback : $auto_failback \n");
    &debug_msg("keepalive     : $keepalive \n");
    &debug_msg("warntime      : $warntime \n");
    &debug_msg("deadtime      : $deadtime \n");
    &debug_msg("initdead      : $initdead \n");
    &debug_msg("ping          : $ping \n");
    &debug_msg("parsed_ha_peer_eth0->: $parsed_ha_peer_eth0 \n");
    &debug_msg("parsed_ha_peer_eth1->: $parsed_ha_peer_eth1 \n");
    
#    %clusterArray = ( 
#		      wfc_timeout => $wfc_timeout,
#		      degr_wfc_timeout=> $degr_wfc_timeout,
#		      rate	=> $rate,
#		      parsed_Std_Pri_fqdn => $pased_Std_Pri_fqdn,
#		      parsed_Std_Sec_fqdn => $parsed_Std_Sec_fqdn,
#		      parsed_Std_Pri_Device => $parsed_Std_Pri_Device,
#		      parsed_Std_Sec_Device => $parsed_Std_Sec_Device,
#		      parsed_Std_Pri_Disk => $parsed_Std_Pri_Disk,
#		      parsed_Std_Sec_Disk => $parsed_Std_Sec_Disk,
#		      parsed_peer_eth1 => $parsed_Std_Sec_ipAddr,
#		      ConStatus => $ConStatus,
#		      SyncStatus => $SyncStatus,
#		      LinkDiagnostic => $LinkDiagnostic,
#		      sync_progress => $sync_progress,
#		      sync_percentage => $sync_percentage,
#		      sync_current	=> $sync_current,
#		      sync_max => $sync_max,
#		      sync_finish => $sync_finish,
#		      sync_speed => $sync_speed,
#		      synchronizing => $synchronizing,
#		      auto_failback => $auto_failback,
#		      keepalive => $keepalive,
#		      warntime => $warntime,
#		      deadtime => $deadtime,
#		      initdead => $initdead,
#		      ping => $ping,
#		      parsed_ha_peer_eth0 => $parsed_ha_peer_eth0,
#		      parsed_ha_peer_eth1 => $parsed_ha_peer_eth1
#		      );
#
    
    $clusterStatusArray = { 
	'ConStatus' => $ConStatus,
	'SyncStatus' => $SyncStatus,
	'LinkDiagnostic' => $LinkDiagnostic,
	'sync_progress' => $sync_progress,
	'sync_percentage' => $sync_percentage,
	'sync_current'	=> $sync_current,
	'sync_max' => $sync_max,
	'sync_finish' => $sync_finish,
	'sync_speed' => $sync_speed,
	'synchronizing' => $synchronizing,
	};
    return $clusterStatusArray;
}

sub validateClusterStatus
{
    $clusterStatusArray = shift;
    $DEBUG = shift;
    if ($DEBUG){ use Sys::Syslog qw( :DEFAULT setlogsock); }

    &check_if_cluster($cce, $DEBUG);

    #DRBD setup validation
    
    if (($clusterStatusArray->{ConStatus} ne "") &&
	($clusterStatusArray->{SyncStatus} ne "") && 
	($clusterStatusArray->{LinkDiagnostic} ne "")) {
	$firstblock = "ok";
	}
    if (($clusterStatusArray->{LinkDiagnostic} ne "Consistent") && 
	($clusterStatusArray->{synchronizing} == "1") && 
	($clusterStatusArray->{sync_progress} ne "") && 
	($clusterStatusArray->{sync_percentage} ne "") &&
        ($clusterStatusArray->{sync_current} ne "") && 
	($clusterStatusArray->{sync_max} ne "") && 
	($clusterStatusArray->{sync_finish} ne "") && 
	($clusterStatusArray->{sync_speed} ne "")) {
	$secondblock = "ok";
    }
    if (($clusterStatusArray->{LinkDiagnostic} == "Consistent") &&
	($clusterStatusArray->{synchronizing} == "0")) {
	$secondblock = "ok";
    }

    &debug_msg("parsed_ha_peer_eth0: $parsed_ha_peer_eth0 \n");
    &debug_msg("parsed_ha_peer_eth1: $parsed_ha_peer_eth1 \n");
    
    &debug_msg("1: $firstblock\n");
    &debug_msg("2: $secondblock\n");

    if (($firstblock ne "ok") || ($secondblock ne "ok")) {
	## Cluster validation failed
	&debug_msg("Cluster validation failed, check the cluster service configuration. \n");
	$cce->bye('FAIL', 'Cluster validation failed, check the cluster service configuration.');
	exit(1);
    }
    return 0;
}

sub updateClusterStatus {
    $clusterStatusArray = shift;
    $cce = shift;
    $DEBUG = shift;
    if ($DEBUG){ use Sys::Syslog qw( :DEFAULT setlogsock); }

    &check_if_cluster($cce, $DEBUG);

    # Global settings
    $my_eth0 = $IP_eth0;
    $my_eth1 = $IP_eth1;
    
    ($clusterOid) = $cce->findx('Cluster_Status');
    &debug_msg("Object already present in CCE. Updating it.\n");

    $clusterStatusObj = { 	
	'ConStatus' => $clusterStatusArray->{ConStatus},
	'SyncStatus' => $clusterStatusArray->{SyncStatus},
	'LinkDiagnostic' => $clusterStatusArray->{LinkDiagnostic},
	'sync_progress' => $clusterStatusArray->{sync_progress},
	'sync_percentage' => $clusterStatusArray->{sync_percentage},
	'sync_current' => $clusterStatusArray->{sync_current},
	'sync_max' => $clusterStatusArray->{sync_max},
	'sync_finish' => $clusterStatusArray->{sync_finish},
	'sync_speed' => $clusterStatusArray->{sync_speed},
	'synchronizing' => $clusterStatusArray->{synchronizing},
    };

    # find the system object
    if (!$clusterOid) {
	($ok) = $cce->create("Cluster_Status", $clusterStatusObj);
    } else {
	($ok) = $cce->set($clusterOid, '', $clusterStatusObj);
    }
    updateCluster::clusterUpdateTimestamp($cce, $DEBUG);
}

sub createClusterConf {
    $cce = shift;
    $my_eth0 = shift;
    $my_eth1 = shift;
    $stand_alone = shift;
    $DEBUG = shift;
    if ($DEBUG){ use Sys::Syslog qw( :DEFAULT setlogsock); }
    &check_if_cluster($cce, $DEBUG);
    &debug_msg("Running createClusterConf\n");

    if ($stand_alone eq "1") {
	&debug_msg("Stand alone cluster node detected: $stand_alone\n");
    }
    else {
	&debug_msg("Cluster with multiple nodes detected: $stand_alone\n");
    }

    # This is where we actually populate CCE with the gathered info:
    ($systemOid) = $cce->findx("System");
    
    ($ok, $systemObj) = $cce->get($systemOid);
    
    $fqdn = $systemObj->{'hostname'} . "." .$systemObj->{'domainname'};
    $defaultGateway = $systemObj->{'gateway'};
    
    # Global settings
    $my_eth0 = $IP_eth0;
    $my_eth1 = $IP_eth1;

    if ($stand_alone eq "1") {
	$my_eth1 = $my_eth0;
	$peer_eth0 = "127.0.0.1";
	$peer_eth1 = "127.0.0.1";
	$Std_Sec_fqdn = "localhost";
	$wfc_timeout = "1";
	$degr_wfc_timeout = "1";
    }
    else {
	$peer_eth0 = "10.255.254.2";
	$peer_eth1 = "10.255.255.2";
	$Std_Sec_fqdn = "slave." . $systemObj->{'domainname'};
	$wfc_timeout = "3";
	$degr_wfc_timeout = "120";
    }

    # DRBD settings
    $Std_Pri_fqdn = $fqdn;
    $on_io_error = "call-local-io-error";
    $rate = "125M";
    $Std_Pri_Device = "/dev/drbd0";
    $Std_Pri_Disk = "/dev/mapper/sysvg-vz";
    $Std_Sec_Device = "/dev/drbd0";
    $Std_Sec_Disk = "/dev/mapper/sysvg-vz";


    # HA settings
    $ping = $defaultGateway;
    $keepalive = "1";
    $warntime = "10";
    $deadtime = "20";
    $initdead = "30";
    $auto_failback = "on";
    
    $clusterObj = {
	'enabled' => '1',
	'my_eth0' => $my_eth0,
	'my_eth1' => $my_eth1,
	'peer_eth0' => $peer_eth0,
	'peer_eth1' => $peer_eth1,
	'Std_Pri_fqdn' => $Std_Pri_fqdn,
	'Std_Pri_Disk' => $Std_Pri_Disk,
	'Std_Pri_Device' => $Std_Pri_Device,
	'wfc_timeout' => $wfc_timeout,
	'degr_wfc_timeout' => $degr_wfc_timeout,
	'on_io_error' => $on_io_error,
	'rate' => $rate,
	'Std_Sec_fqdn' => $Std_Sec_fqdn,
	'Std_Sec_Device' => $Std_Sec_Device,
	'Std_Sec_Disk' => $Std_Sec_Disk,
	'ping' => $ping,
	'keepalive' => $keepalive,
	'warntime' => $warntime,
	'deadtime' => $deadtime,
	'initdead' => $initdead,
	'auto_failback' => $auto_failback,
	'initialdrbd' => $Std_Pri_fqdn,
	'makeprimary' => "1",
	'stand_alone' => $stand_alone
	};
    $cce->create("Cluster", $clusterObj);
    updateCluster::clusterUpdateTimestamp($cce, $DEBUG);
}

sub createOldClusterConf {
    $cce = shift;
    $my_eth0 = shift;
    $my_eth1 = shift;
    $DEBUG = shift;
    if ($DEBUG){ use Sys::Syslog qw( :DEFAULT setlogsock); }

    &check_if_cluster($cce, $DEBUG);

    # This is where we actually populate CCE with the gathered info:
    ($systemOid) = $cce->findx("System");
    
    ($ok, $systemObj) = $cce->get($systemOid);
    
    $fqdn = $systemObj->{'hostname'} . "." .$systemObj->{'domainname'};
    $defaultGateway = $systemObj->{'gateway'};
    
# Global settings
    $my_eth0 = $IP_eth0;
    $my_eth1 = $IP_eth1;
    
    $peer_eth0 = "10.255.254.2";
    $peer_eth1 = "10.255.255.2";
	
# DRBD settings
    $Std_Pri_fqdn = $fqdn;
    $Std_Sec_fqdn = "slave." . $systemObj->{'domainname'};
    $wfc_timeout = "3";
    $degr_wfc_timeout = "120";
    $on_io_error = "call-local-io-error";
    $rate = "125M";
    $Std_Pri_Device = "/dev/drbd0";
    $Std_Pri_Disk = "/dev/mapper/sysvg-vz";
    $Std_Sec_Device = "/dev/drbd0";
    $Std_Sec_Disk = "/dev/mapper/sysvg-vz";
    
    
# HA settings
    $ping = $defaultGateway;
    $keepalive = "1";
    $warntime = "10";
    $deadtime = "20";
    $initdead = "30";
#----------------------------------------------------
# Create /etc/ha.d/authkeys:
#----------------------------------------------------
    system('/bin/echo "auth 1" > /etc/ha.d/authkeys');
    system('/bin/echo "1 sha1 ba1a0733943194f5108a4a474bd2344a" >> /etc/ha.d/authkeys');
    system("chmod 600 /etc/ha.d/authkeys");
    
# Create /etc/ha.d/haresources:
#----------------------------------------------------
    system("/bin/echo \"$Std_Pri_fqdn drbddisk::vz Filesystem::/dev/drbd0::/vz::ext3 WatchDog libvirtd libvirt-guests vzeventd vz\" > /etc/ha.d/haresources");
    
    $clusterObj = {
	'enabled' => '0',
	'my_eth0' => $my_eth0,
	'my_eth1' => $my_eth1,
	'peer_eth0' => $peer_eth0,
	'peer_eth1' => $peer_eth1,
	'Std_Pri_fqdn' => $Std_Pri_fqdn,
	'Std_Pri_Disk' => $Std_Pri_Disk,
	'Std_Pri_Device' => $Std_Pri_Device,
	'wfc_timeout' => $wfc_timeout,
	'degr_wfc_timeout' => $degr_wfc_timeout,
	'on_io_error' => $on_io_error,
	'rate' => $rate,
	'Std_Sec_fqdn' => $Std_Sec_fqdn,
	'Std_Sec_Device' => $Std_Sec_Device,
	'Std_Sec_Disk' => $Std_Sec_Disk,
	'ping' => $ping,
	'keepalive' => $keepalive,
	'warntime' => $warntime,
	'deadtime' => $deadtime,
	'initdead' => $initdead
	};
    $cce->create("Cluster", $clusterObj);
    system("/usr/bin/yes yes | /sbin/drbdadm create-md all > /dev/null 2>&1");
    system("/etc/init.d/drbd start > /dev/null 2>&1");
#    system("/sbin/drbdadm -- --overwrite-data-of-peer primary all > /dev/null 2>&1");
    system("/sbin/drbdadm disconnect all > /dev/null 2>&1");
    system("/sbin/drbdadm connect all > /dev/null 2>&1");
    
    updateCluster::clusterUpdateTimestamp($cce, $DEBUG);
}

sub clusterUpdateTimestamp {
    $cce = shift;
    $DEBUG = shift;
    if ($DEBUG){ use Sys::Syslog qw( :DEFAULT setlogsock); }

    &check_if_cluster($cce, $DEBUG);

    @oids = $cce->find('Cluster_Update');
    if ($#oids < 0) {
	&debug_msg("Object not yet in CCE. Creating new one.\n");
	($ok) = $cce->create('Cluster_Update', {
	    'timestamp' => time()
	    });
    } else {
	&debug_msg("Object already present in CCE. Updating it.\n");
	# find the system object
	($clusterUpdateOid) = $cce->find('Cluster_Update');
	($ok, $ClusterUpdateObj) = $cce->get($clusterUpdateOid);
	
	($ok) = $cce->set($clusterUpdateOid, '',{        
	    'timestamp' => time()
	    });
    }
}

sub debug_msg { 
    if ($DEBUG) {
        if ($DEBUG){ use Sys::Syslog qw( :DEFAULT setlogsock); }
        my $msg = shift;
        $user = $ENV{'USER'};
        setlogsock('unix');
        openlog($0,'','user');
        syslog('info', "$ARGV[0]: $msg");
        closelog;
    }
}

sub check_if_cluster {

    $cce = shift;
    $DEBUG = shift;
    if ($DEBUG){ use Sys::Syslog qw( :DEFAULT setlogsock); }
    &debug_msg("Running check_if_cluster \n");
        
    # Poll 'System' Object:
    ($systemOid) = $cce->findx("System");
        
    ($ok, $systemObj) = $cce->get($systemOid);
    
    $isLicenseAccepted = $systemObj->{'isLicenseAccepted'};

    $etcvz_is_link = 0;
    if (-l "/etc/vz") {
        &debug_msg("Directory /etc/vz is a symbolic link - good.\n");
	$etcvz_is_link = 1;
    }

    if (($isLicenseAccepted eq "0") && ($etcvz_is_link eq "1")) {
        &debug_msg("Bypassing any DRBD related tasks as web based setup is not yet finished. \n");
        $cce->bye('SUCCESS', 'Bypassing any DRBD related tasks as web based setup is not yet finished.');
        exit(0)
    }
    if ($etcvz_is_link eq "0") {
        &debug_msg("Bypassing any DRBD related tasks as this does not appear to be a cluster ready box. \n");
        $cce->bye('SUCCESS', 'Bypassing any DRBD related tasks as this does not appear to be a cluster ready box.');
        exit(0)
    }

}

1;
